#include "http_codec_mec.hh"

#include "loggers.hh"

bool http_codec_mec::encode_body_json(const LibHttp__JsonMessageBodyTypes::JsonBody &p_json_body, OCTETSTRING &p_encoding_buffer, const std::string &p_content_type) {
  loggers::get_instance().log(">>> http_codec_mec::encode_body_json");

  std::map<std::string, std::unique_ptr<codec_gen<Record_Type, Record_Type> > >::const_iterator it;
  bool processed = false;
  loggers::get_instance().log("http_codec_mec::encode_body_json: Content-Type:'%s'", p_content_type.c_str());
  if (p_content_type.find("json") != std::string::npos) {
    it = _codecs.find("json"); // TODO Use params
    if (it != _codecs.cend()) {
      loggers::get_instance().log("http_codec_mec::encode_body_json: Call 'json_codec'");
      _codecs["json"]->encode((Record_Type&)p_json_body, p_encoding_buffer); // TODO Use params
      processed = true;
    }
  } // TODO Add new HTTP message codec here
  if (!processed) {
    loggers::get_instance().warning("http_codec_mec::encode_body_json: Unsupported HTTP codec, use raw field as default");
    p_encoding_buffer = OCTETSTRING(0, nullptr);
  }

  return true;
}

bool http_codec_mec::decode_body_json(const OCTETSTRING &p_data, LibHttp__JsonMessageBodyTypes::JsonBody &p_json_body, const std::string &p_content_type, params* p_params) {
  loggers::get_instance().log(">>> http_codec_mec::decode_body_json");

  if (_codecs["json"].get() != nullptr) {
    loggers::get_instance().log("http_codec::decode_body_json: Call 'json_codec'");
    if (_codecs["json"]->decode(p_data, (Record_Type&)p_json_body, p_params) == -1) {
      loggers::get_instance().warning("http_codec::decode_body_json: Failed to decode JSON message");
      p_json_body.raw() = CHARSTRING(p_data.lengthof(), (char*)static_cast<const unsigned char*>(p_data));
    } else {
      loggers::get_instance().log_msg("http_codec::decode_body_json: Decoded message:", p_json_body);
    }
  } else {
      loggers::get_instance().error("http_codec::decode_body_json: Codec not found");
      return false;
  }

  return true;
}
